<?php

/**
 * @file
 */

namespace Drupal\content_snapshot\Normalizer;

use Drupal\content_snapshot\Encoder\YamlEncoder;
use Drupal\content_snapshot\Importer\ImporterInterface;
use Drupal\content_snapshot\Logger\LoggerInterface;
use Drupal\content_snapshot\ReferenceRestorer\ReferenceRestorerInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\serialization\Normalizer\EntityReferenceFieldItemNormalizer as BaseEntityReferenceFieldItemNormalizer;

/**
 * Class EntityReferenceFieldItemNormalizer
 */
class EntityReferenceFieldItemNormalizer extends BaseEntityReferenceFieldItemNormalizer {

  use ContentSnapshotNormalizerContextTrait;

  protected $format = YamlEncoder::FORMAT;

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $config;

  /**
   * @var \Drupal\content_snapshot\Logger\LoggerInterface
   */
  private $logger;


  /**
   * {@inheritDoc}
   */
  public function __construct(EntityRepositoryInterface $entity_repository, ImmutableConfig $config, LoggerInterface $logger) {
    parent::__construct($entity_repository);
    $this->config = $config;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   *
   * @param $field \Drupal\Core\Field\FieldItemInterface
   */
  public function normalize($field, $format = NULL, array $context = []) {

    $this->validateContext($context);

    $attributes = parent::normalize($field, $format, $context);

    $this->cleanupNormalization($attributes);

    return $attributes;
  }

  /**
   * {@inheritdoc}
   */
  public function denormalize($data, $class, $format = NULL, array $context = []) {

    $this->validateContext($context);

    /** @var \Drupal\Core\Field\FieldItemInterface $targetInstance */
    $targetInstance = $context['target_instance'];
    /** @var \Drupal\Core\Field\FieldItemListInterface $fieldListItem */
    $fieldListItem = $targetInstance->getParent();
    /** @var \Drupal\Core\Entity\FieldableEntityInterface $entity */
    $entity = $fieldListItem->getParent()->getValue();

    if (!empty($context[ReferenceRestorerInterface::RESTORING_REFERENCES_CONTEXT])) {
      return $this->referenceRestoringDenormalize($data, $class, $format, $context, $targetInstance, $fieldListItem, $entity);
    }

    return $this->initialDenormalize($data, $class, $format, $context, $targetInstance, $fieldListItem, $entity);
  }

  /**
   * @param $data
   * @param $class
   * @param $format
   * @param array $context
   * @param \Drupal\Core\Field\FieldItemInterface $targetInstance
   * @param \Drupal\Core\Field\FieldItemListInterface $fieldListItem
   * @param \Drupal\Core\Entity\FieldableEntityInterface $entity
   *
   * @throws \Drupal\Core\TypedData\Exception\ReadOnlyException
   */
  private function initialDenormalize($data, $class, $format, array $context, FieldItemInterface $targetInstance, FieldItemListInterface $fieldListItem, FieldableEntityInterface $entity) {
    // This is workaround for some uid fields that are required for
    // entity to be saved.
    if ($entity->getEntityType()->getKey('owner') === $fieldListItem->getName()) {
      $targetInstance->setValue($this->config->get('default_owner_id'));
    }

    // We want entity reference fields to be empty during the first phase.
    // References will be restored later.
    return NULL;
  }

  /**
   * @param $data
   * @param $class
   * @param $format
   * @param array $context
   * @param \Drupal\Core\Field\FieldItemInterface $targetInstance
   * @param \Drupal\Core\Field\FieldItemListInterface $fieldListItem
   * @param \Drupal\Core\Entity\FieldableEntityInterface $entity
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\TypedData\Exception\ReadOnlyException
   */
  private function referenceRestoringDenormalize($data, $class, $format, array $context, FieldItemInterface $targetInstance, FieldItemListInterface $fieldListItem, FieldableEntityInterface $entity) {
    $fieldType = $targetInstance->getFieldDefinition()->getType();
    $fieldName = $fieldListItem->getName();
    $entityTypeId = $entity->getEntityTypeId();
    $parentEntityId = $entity->id();
    $parentUuid = $entity->uuid();

    if ($fieldType === 'entity_reference_revisions') {
      $targetEntity = $this->entityRepository->loadEntityByUuid($data['target_type'], $data['target_uuid']);

      if (!$targetEntity) {
        $this->logger->warning("Field '$fieldName' of the type '$fieldType' of an entity '$entityTypeId' with id '$parentEntityId' and UUID '$parentUuid' have broken reference that cannot be restored.");
        return NULL;
      }
      // Restore required property.
      $data['target_revision_id'] = $targetEntity->getRevisionId();
    }

    // Now that we have all entities in the database, we can denormalize all
    // entity reference fields normally.
    return parent::denormalize($data, $class, $format, $context);
  }


  /**
   * @param array $attributes
   */
  protected function cleanupNormalization(array &$attributes): void {
    // We don't want to store ids that are database dependent if we have uuid
    // available.

    // Some entities such as shortcuts have non-numeric target_ids that we
    // cannot drop.
    if (!empty($attributes['target_uuid']) && is_numeric($attributes['target_id'])) {
      unset($attributes['target_id']);
    }

    unset($attributes['url']);

    // This can be provided by the entity_reference_revisions field.
    unset($attributes['target_revision_id']);
  }
}
