<?php

/**
 * @file
 */

namespace Drupal\content_snapshot\Normalizer;

use Drupal\content_snapshot\Encoder\YamlEncoder;
use Drupal\content_snapshot\Importer\ImporterInterface;
use Drupal\content_snapshot\ReferenceRestorer\ReferenceRestorerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\link\Plugin\Field\FieldType\LinkItem;

/**
 * Class LinkFieldItemNormalizer.
 */
class LinkFieldItemNormalizer extends \Drupal\serialization\Normalizer\FieldItemNormalizer {

  use ContentSnapshotNormalizerContextTrait;

  protected $format = YamlEncoder::FORMAT;

  protected $supportedInterfaceOrClass = LinkItem::class;

  private const FIELD_METADATA_KEY = '_content_snapshot';
  private const TARGET_SCHEME_KEY = 'scheme';
  private const TARGET_ENTITY_TYPE_KEY = 'target_type';
  private const TARGET_ENTITY_UUID_KEY = 'target_uuid';
  private const TARGET_ENTITY_ID_KEY = 'target_id';

  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   *
   * @param $field \Drupal\Core\Field\FieldItemInterface
   */
  public function normalize($field, $format = NULL, array $context = []) {

    $this->validateContext($context);

    $attributes = parent::normalize($field, $format, $context);

    $uri = $attributes['uri'];

    $entityPrefix = 'entity:';
    if (strpos($uri, $entityPrefix) === 0) {
      $params = Url::fromUri($uri)->getRouteParameters();
      $entityType = key($params);

      $entity = $this->entityTypeManager->getStorage($entityType)->load($params[$entityType]);
      if ($entity) {
        $id = $entity->uuid() ?? $entity->id();

        $attributes[self::FIELD_METADATA_KEY] = [
          self::TARGET_SCHEME_KEY => 'entity',
          self::TARGET_ENTITY_TYPE_KEY => $entity->getEntityTypeId(),
        ];

        // @todo note that currently content without uuid is filtered out.
        if ($uuid = $entity->uuid()) {
          $attributes[self::FIELD_METADATA_KEY][self::TARGET_ENTITY_UUID_KEY] = $uuid;
        } else {
          $attributes[self::FIELD_METADATA_KEY][self::TARGET_ENTITY_ID_KEY] = $id;
        }

        unset($attributes['uri']);

        return $attributes;
      }
    }

    return $attributes;
  }

  /**
   * {@inheritDoc}
   */
  public function denormalize($data, $class, $format = NULL, array $context = []) {

    $this->validateContext($context);

    if (empty($data[self::FIELD_METADATA_KEY])) {
      return parent::denormalize($data, $class, $format, $context);
    }

    $metadata = $data[self::FIELD_METADATA_KEY];
    $scheme = $metadata[self::TARGET_SCHEME_KEY];

    // Initial import
    if (empty($context[ReferenceRestorerInterface::RESTORING_REFERENCES_CONTEXT])) {
      // We are temporarily replacing entity references by the homepage urls.
      // They will be replaced with correct references later, during the
      // references restoring phase.
      if ($scheme === 'entity') {
        $data['uri'] = 'internal:/';
      }

      return parent::denormalize($data, $class, $format, $context);
    }

    if (!empty($data[self::FIELD_METADATA_KEY])) {

      if ($scheme === 'entity') {
        $entityTypeId = $metadata[self::TARGET_ENTITY_TYPE_KEY];
        if (!empty($metadata[self::TARGET_ENTITY_UUID_KEY])) {
          $uuid = $metadata[self::TARGET_ENTITY_UUID_KEY];
          $storage = $this->entityTypeManager->getStorage($entityTypeId);

          $entities = $storage->loadByProperties(['uuid' => $uuid]);
          if (!empty($entities)) {
            $entity = $entities[key($entities)];
            $data['uri'] = 'entity:' . $entity->toUrl()->getInternalPath();
          }
        }
      }
    }

    return parent::denormalize($data, $class, $format, $context);
  }

}
