<?php

/**
 * @file
 */

namespace Drupal\content_snapshot\Normalizer;

use Drupal\content_snapshot\Importer\ImporterInterface;
use Drupal\content_snapshot\Logger\LoggerInterface;
use Drupal\content_snapshot\ReferenceRestorer\ReferenceRestorerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\StringItem;
use Drupal\Core\Url;
use Drupal\serialization\Normalizer\FieldItemNormalizer;

/**
 * Class StringFieldItemNormalizer.
 */
class StringFieldItemNormalizer extends FieldItemNormalizer {

  use ContentSnapshotNormalizerContextTrait;

  protected $format = 'yaml_content_snapshot';

  protected $supportedInterfaceOrClass = StringItem::class;

  private const TARGET_TYPE_KEY = '_content_snapshot_target_type';
  private const TARGET_UUID_KEY = '_content_snapshot_target_uuid';

  /**
   * @var \Drupal\content_snapshot\Logger\LoggerInterface
   */
  private $logger;

  /**
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  private $entityRepository;

  /**
   * StringFieldItemNormalizer constructor.
   *
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   * @param \Drupal\content_snapshot\Logger\LoggerInterface $logger
   */
  public function __construct(EntityRepositoryInterface $entityRepository, LoggerInterface $logger) {
    $this->logger = $logger;
    $this->entityRepository = $entityRepository;
  }

  /**
   * {@inheritDoc}
   *
   * @param \Drupal\Core\Field\FieldItemInterface $fieldItem
   */
  public function normalize($fieldItem, $format = NULL, array $context = []) {

    $this->validateContext($context);

    /** @var \Drupal\Core\Field\FieldItemListInterface $fieldItemList */
    $fieldItemList = $fieldItem->getParent();

    /** @var \Drupal\Core\Entity\FieldableEntityInterface $entity */
    $entity = $fieldItemList->getParent()->getValue();

    $normalized = parent::normalize($fieldItem, $format, $context);

    if ($entity->getEntityTypeId() === 'path_alias' && $fieldItemList->getName() === 'path') {

      $params = Url::fromUri("internal:" . $normalized['value'])->getRouteParameters();
      $entity_type = key($params);

      $entity = $this->entityRepository->getCanonical($entity_type, $params[$entity_type]);
      if (!$entity) {
        // @todo We should deal with this more softly - by simply preventing
        // path_alias from being snapshotted, and displaying warning/error.
        throw new \LogicException("Path data in the path_alias '" . $entity->uuid() . "' couldn't be normalized.");
      }

      unset($normalized['value']);
      $normalized[static::TARGET_TYPE_KEY] = $entity_type;
      $normalized[static::TARGET_UUID_KEY] = $entity->uuid();
    }

    return $normalized;
  }

  /**
   * {@inheritDoc}
   */
  public function denormalize($data, $class, $format = NULL, array $context = []) {

    $this->validateContext($context);

    /** @var \Drupal\Core\Field\FieldItemInterface $targetInstance */
    $targetInstance = $context['target_instance'];
    $fieldItemList = $targetInstance->getParent();
    /** @var \Drupal\Core\Entity\FieldableEntityInterface $entity */
    $entity = $fieldItemList->getParent()->getValue();

    if ($entity->getEntityTypeId() === 'path_alias' && $fieldItemList->getName() === 'path') {

      // Field can be filled witch correct content only during references
      // restoring phase.
      if (empty($context[ReferenceRestorerInterface::RESTORING_REFERENCES_CONTEXT])) {
        return NULL;
      }

      if (!isset($data[static::TARGET_TYPE_KEY], $data[static::TARGET_UUID_KEY])) {
        throw new \LogicException("Path data is missing in the path_alias '" . $entity->uuid());
      }

      $targetEntity = $this->entityRepository->loadEntityByUuid($data[static::TARGET_TYPE_KEY], $data[static::TARGET_UUID_KEY]);

      if (!$targetEntity) {
        return NULL;
      }

      $data['value'] = '/' . $targetEntity->toUrl()->getInternalPath();
      unset($data[static::TARGET_TYPE_KEY], $data[static::TARGET_UUID_KEY]);
    }


    return parent::denormalize($data, $class, $format, $context);
  }

}
